%macro do_maxT(
use_lib=WORK,
/*********************************/
/*** graphic/ reports parameters ***/
title1=Results of Max Test of Freidlin et al. followed by max T p-value adjustment,
	/*** User can put in more informative titles ***/
title2=MAX-maxT results,
get_thin_ds=T,
	/*** Flag telling if thin data set having only max_abs_zs, pval and locus
	should be out to text file all ***/
get_graphic=T, 
	/*** Flag telling if graphic should be produced. ***/
ps_graphic=40, ls_graphic=90,
	/*** Params controlling pagesize and linesize for graphic ***/
get_reports=T,
	/*** Flag telling if reports should be produced. ***/
ps_reports=33, ls_reports=90,
	/*** Params controlling pagesize and linesize for reports ***/
top_SNPs=25,
	/*** Print out a list of the top top_SNPs SNPs. Changing top_SNPs to a number
	less than 1 suppresses this output ***/
ps_top_SNPs=33, ls_top_SNPs=90
);
/*** Get names of files to read and how many columns
each file uses. ***/
/****************************************************************/

/*** Note that BIG ASSUMPTION here is that MAX_1 in the first (top) data
set listed in  &use_lib..output_file_names is the actual max_abs_zs, i.e. the
one from the observed data. ***/

/****************************************************************/
data tmp; set &use_lib..output_file_names end=last;
call symput("file_" || TRIM(LEFT(_n_)), output_file);
call symput("no_resamps_" || TRIM(LEFT(_n_)), num_resamps);
if last then call symput('_no_of_output_files_', _n_);
run;
/***********************************************************************/
/*** Create some useful MACRO variables ***/
%let start_max_no_1=1;
%let end_max_no_1=&no_resamps_1;
%let total_resamps=&no_resamps_1;
%let j=1;

%do i=2 %to &_no_of_output_files_;
	%let start_max_no_&i=%eval(&&end_max_no_&j+1);
	%let end_max_no_&i=%eval(&&end_max_no_&j+&&no_resamps_&i);
	%let j=%eval(&j+1);
	%let total_resamps=%eval(&total_resamps+&&no_resamps_&i);
%end;
/***********************************************************************/
/*** Read in the data. ***/
/*** Note that &file_i will have as a prefix the library name ***/
/****************************************************************/

/*** We say it again:
	Note that BIG ASSUMPTION here is that MAX_1 in the first (top) data
set listed in  &use_lib..output_file_names is the actual max_abs_zs, i.e. the
one from the observed data. ***/

/****************************************************************/
DATA ALL; SET &file_1(rename=(max_1=max_abs_zs
%do j=2 %to %eval(&no_resamps_1+1); 
		max_&j.=max_%eval(&j-1) 
	%end;
)); run;
proc sort; by locus_no; run;

%do i=2 %to &_no_of_output_files_;
proc sort data=&&file_&i.; by locus_no; run;

data all; 
merge all &&file_&i.(rename=(
	%do j=1 %to &&no_resamps_&i; 
		max_%eval(&&no_resamps_&i+1-&j)=max_%eval(&&start_max_no_&i+&&no_resamps_&i-&j) 
	%end; 
	));
by locus_no;
run;
%end;
/***********************************************************************/
/*** Start maxT algorithm ***/
proc sort; by max_abs_zs; run;

data all(keep=locus_no max_abs_zs pval); set all;
retain %do i=1 %to &total_resamps; u_&i  %end; ;
count=0;
if _n_=1 then do;
	 %do i=1 %to &total_resamps; 
		u_&i=max_&i ; 
		if u_&i>=max_abs_zs then count=count+1;
	%end; 
end;
else do;
	%do i=1 %to &total_resamps; 
		u_&i=max(max_&i, u_&i) ; 
		if u_&i>=max_abs_zs then count=count+1;
	%end;
end;
pval=count/&total_resamps;
run;

proc sort; by descending max_abs_zs; run;
/*** Enforce monotonicity ***/
/*** Note that a variable, pval, is input while a variable, p_val, is output ***/
data all(drop=pval rename=(adj_p=p_val)); set all;
retain adj_p;
if _n_=1 then adj_p=pval;
if _n_>1 then adj_p=max(pval, adj_p);
run;
/************************************************************/

/*** max T alg is finished. Output results ***/

/************************************************************/
title1 "&title1";
title2 "&title2";
/*** Put SNP names onto DS all ***/
/*** Put adjusted p-values back onto max_data ***/
proc sort; by locus_no; run;
proc sort data=&use_lib..locus_key; by locus_no; run;
proc sort data=&use_lib..max_data; by locus_no; run;
data all; merge all &use_lib..locus_key; by locus_no; run;

data &use_lib..max_data; merge &use_lib..max_data all; 
by locus_no; run;

proc sort; by descending max_abs_zs; run;
data &use_lib..all; set all(drop=locus_no); run;


/*** Print out 'thin' data set ***/
%if &get_thin_ds=T %then %do;
      data _null_;
      set  library.ALL                                     end=EFIEOD;
      %let _EFIERR_ = 0; /* set the ERROR detection macro variable */
      %let _EFIREC_ = 0;     /* clear export record count macro variable */
      file 'all' delimiter=' ' DSD DROPOVER lrecl=32767;
         format locus $33. ;
         format max_abs_zs best12. ;
         format p_val best12. ;
      if _n_ = 1 then        /* write column names */
       do;
         put
         'locus'
         ' '
         'max_abs_zs'
         ' '
         'p_val'
         ;
       end;
       do;
         EFIOUT + 1;
         put locus $ @;
         put max_abs_zs @;
         put p_val ;
         ;
       end;
      if _ERROR_ then call symputx('_EFIERR_',1);  /* set ERROR detection macro variable */
      if EFIEOD then call symputx('_EFIREC_',EFIOUT);
      run;
%end; /*** end of %if &get_thin_ds=T %then %do ***/
/*******************************************************/
/**** Do 2 reports: Report 1 shows the CC stat vs genotype table for each
locus. Report 2 shows the results of the MAX test. ***/
%if &get_reports=T %then %do;
title3 'Numbers used in CC Stat by Genotype Table';
title4 "(Prior to add'n of 1/2 to each cell)"; 
options ps=&ps_reports ls=&ls_reports;

data tmp(keep=locus c0 c1 c2 rowtot rowname MAF); 
set &use_lib..max_data(keep=locus r0 r1 r2 s0 s1 s2 n1 n2 n);
/*** Note that we formerly added 1/2 to each of these values ***/
	MAF=(n1+2*n2-1.5)/(2*(N-3));
	c0=r0-0.5; c1=r1-0.5; c2=r2-0.5;
	rowtot=c0+c1+c2;
	rowname='Case';
	output;
	c0=s0-0.5; c1=s1-0.5; c2=s2-0.5;
	rowtot=c0+c1+c2;
	rowname='Control';
	output;
	run;

proc report data=tmp nofs split="*"; 
column locus MAF ('-Genotype Distribution Table-*Minor Allele Count' 
	rowname c0 c1 c2 rowtot);
define locus / group;
define MAF / group format=3.2 'MAF*---';
define rowname / display width=7 'CC stat*---';
define c0 / analysis sum format=7. 'Count=0*---';
define c1 / analysis sum  format=7. 'Count=1*---';
define c2 / analysis sum  format=7. 'Count=2*---';
define rowtot / analysis sum format=7.  'Row Tot*---';
break after locus / ol summarize skip suppress;
run; quit;

title3 "A total of &total_resamps permutations of the CC Stat vector were performed.";
proc report data=&use_lib..max_data(keep=locus z_0 z_05 z_1 max_abs_zs p_val) nofs split="*"; 
column locus ('-Details of MAX test-' 
('-Components-' z_0 z_05 z_1) max_abs_zs p_val);
define locus / group ;
define z_0 / display format=7.2 'Rec';
define z_05 / display format=7.2 'Add';
define z_1 / display format=7.2 'Dom';
define max_abs_zs / display format=7.2 'MAX';
define p_val / display format=11.5 'max T*adjusted*p-val(MAX)';
run; quit;
%end; /*** end of "%if &get_reports=T ***/
/*********************************************************************/
/*** Try to give an idea of population substructure or of true discovery rate ***/
/*** Show plot of Z_0 vs MAF. Should see that variance is roughly constant if there's 
little pop substructure. ***/
%if &get_graphic=T %then %do;
proc plot data=&use_lib..max_data(keep=z_0 frac_min) nolegend;
   plot z_0*frac_min;
	title3 'Plot of Z_0 vs MAF';
	title4 'If population substructure is minimal then variance of';
	title5 'Z_0 should be constant across MAF.';
run;
quit;
%end; /** end of "%if get_graphic_=T %then %do ***/
/*********************************************************************/
/*** Now print out top SNPs ***/
%if &top_SNPs>=1 %then %do;
options ps=&ps_top_SNPs ls=&ls_top_SNPs;
proc sort data=&use_lib..max_data(keep=locus p_val z_0 z_05 z_1 max_abs_zs) out=tmp; 
by p_val; run;

title3 "A total of &total_resamps permutations of the CC Stat vector were performed.";
title4 "List of &top_SNPs. SNPs most likely to be associated with the phenotype.";
proc report data=tmp(obs=&top_SNPs.) nofs split="*"; 
column locus ('-Details of MAX test-' 
('-Components-' z_0 z_05 z_1) max_abs_zs p_val);
define locus / display ;
define z_0 / display format=7.2 'Rec';
define z_05 / display format=7.2 'Add';
define z_1 / display format=7.2 'Dom';
define max_abs_zs / display format=7.2 'MAX';
define p_val / display format=11.5 'max T*adjusted*p-val(MAX)';
run; quit;
%end; /*** end of %if &top_SNPs>=1 %then %do; ***/
/*** Clean-up ***/
proc datasets; delete tmp ; run; quit;
/*** End of section outputting final results for maxT test ***/
/*************************************************************/

%MEND;











